: # use perl
        eval 'exec perl -S $0 "$@"'
                if $runnning_under_some_shell;

#
# test.pl - regression tests for weblint
#
# Copyright (C) 1995 Neil Bowers.  All rights reserved.
#
# See README for additional blurb.
# Bugs, comments, suggestions welcome: neilb@khoral.com
#
$VERSION  = '1.004';
($PROGRAM = $0) =~ s@.*/@@;
$FILENAME = 'testfile.html';
$LOGFILE  = 'weblint-test.log';
$ENV{WEBLINTRC} = '/dev/null';


&WeblintTestInitialize();


&ExpectOK('simple syntactically correct html', '',
	  "<HTML>\n<HEAD><TITLE>test</TITLE></HEAD>\n".
	  "<BODY>this is the body</BODY>\n</HTML>");

&ExpectOK('paragraph usage', '',
	  "<HTML>\n<HEAD><TITLE>test</TITLE></HEAD>\n".
          "<BODY>first paragraph<P>second paragraph</BODY>\n</HTML>");

&ExpectOK('html which starts with DOCTYPE specifier', '',
	  "<!DOCTYPE HTML PUBLIC '-//W3O//DTD WWW HTML 2.0//EN'>\n".
	  "<HTML>\n<HEAD><TITLE>test</TITLE></HEAD>\n".
	  "<BODY>this is the body</BODY>\n</HTML>");

&ExpectOK('acceptable usage of META element', '',
	  '<HTML><HEAD><TITLE>foo</TITLE>'.
	  '<META NAME="IndexType" CONTENT="Service"></HEAD>'.
	  '<BODY>this is the body</BODY></HTML>');

&ExpectOK('correct use of information type and font style elements', '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	  "<EM>Emphasized Text</EM>\n".
	  "<CITE>Cited Text</CITE>\n".
	  "<STRONG>Strongly emphasized Text</STRONG>\n".
	  "<CODE>Teletype Text</CODE>\n".
	  "<SAMP>sequence of literal characters</SAMP>\n".
	  "<KBD>Keyboarded Text</KBD>\n".
	  "<VAR>Variable name</VAR>\n".
	  "<DFN>Defining instance</DFN>\n".
	  "<Q>Short quotation</Q>\n".
	  "<LANG>alter language context</LANG>\n".
	  "<AU>Name of author</AU>\n".
	  "<PERSON>name of a person</PERSON>\n".
	  "<ACRONYM>acronym</ACRONYM>\n".
	  "<ABBREV>Abbreviation</ABBREV>\n".
	  "<INS>Inserted text</INS>\n".
	  "<DEL>Deleted text</DEL>\n".
	  "<B>Bold text</B>\n".
	  "<I>Italic text</I>\n".
	  "<TT>Teletype text</TT>\n".
	  "<U>Underlined text</U>\n".
	  "<S>Striked through text</S>\n".
	  "<BIG>Big text</BIG>\n".
	  "<SMALL>Small text</SMALL>\n".
	  "<SUB>Subscript text</SUB>\n".
	  "<SUP>Superscript text</SUP>\n".
	  '</BODY></HTML>');

&ExpectOK('IMG element with ALT and ISMAP attributes', '',
	  '<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>'.
	  '<IMG SRC=foo.gif ISMAP ALT="alt text">'.
	  '</BODY></HTML>');

&ExpectOK('newline within a tag', '',
	  '<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>'.
	  '<IMG SRC="foo.gif"'."\n".'ALT="alt text">'.
	  '</BODY></HTML>');

&ExpectOK('simple comment', '',
	  "<!-- comment before the HTML element -->\n".
	  "<HTML>\n".
	  "<!-- comment between the HTML and HEAD elements -->\n".
	  "<HEAD>\n".
	  "<!-- comment in the HEAD element -->\n".
	  "<TITLE>foo</TITLE></HEAD><BODY>".
	  "<!-- this is a simple comment in the body -->\n".
	  "this is the body\n".
	  "</BODY>\n".
	  "<!-- comment between end of BODY and end of HTML -->\n".
	  "</HTML>\n".
	  "<!-- comment after the end of the HTML element -->\n");

&ExpectOK('comment with space before the closing >', '',
	  '<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>'.
	  '<!-- this is a simple comment -- >'.
	  'this is the body'.
	  '</BODY></HTML>');

&ExpectOK('whitespace around the = of an element attribute', '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>".
	  "<IMG SRC = foo.gif ALT=\"alt text\">".
	  "</BODY></HTML>");

&ExpectOK('legal unordered list', '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	  "<UL>\n".
	  "<LI>first item\n".
	  "<LI>second item</LI>\n".
	  "</UL>\n".
	  "</BODY></HTML>\n");

&ExpectOK('legal definition list', '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	  "<DL>\n".
	  "<DT>first tag<DD>first definition\n".
	  "<DT>second tag<DD>second definition\n".
	  "</DL>\n".
	  "</BODY></HTML>\n");

&ExpectOK('simple table', '',
	  '<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>'.
	  '<TABLE><TR><TH>height<TD>1.0<TR><TH>weight<TD>1.0</TABLE>'.
	  '</BODY></HTML>');

&ExpectFAIL('table without TR', '',
	    "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	    "<TABLE><TH>height<TD>1.0<TR><TH>weight<TD>1.0</TABLE>".
	    "</BODY></HTML>",
	    2, 'required-context',
	    2, 'required-context');

&ExpectOK('simple figure with caption', '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	  "<FIG SRC=\"nicodamus.jpeg\">\n".
          "<CAPTION>Ground dweller: <I>Nicodamus bicolor</I>\n".
	  "builds silk snares</CAPTION>\n".
	  "<P>A small hairy spider light fleshy red in color with a brown ".
	  "abdomen.\n<CREDIT>J. A. L. Cooke/OSF</CREDIT>\n".
	  "</FIG></BODY></HTML>");

&ExpectOK('simple math usage', '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	  "<MATH>\n".
	  "<BOX>1+x<OVER>sin x</BOX>\n".
	  "<ABOVE>1+x</ABOVE>\n".
	  "<BELOW>1+x</BELOW>\n".
	  "<SQRT>1+x</SQRT>\n".
	  "<ROOT>3<OF>1+x</ROOT>\n".
	  "<ARRAY><ROW><ITEM>a<SUB>11</SUB><ITEM>a<SUB>12</SUB></ARRAY>\n".
	  "</MATH>\n".
	  "</BODY></HTML>\n");

&ExpectFAIL('no HTML tags around document', '',
	    "<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>this is the body</BODY>\n",
	    1, 'html-outer',
	    1, 'must-follow');

&ExpectFAIL('whitespace between opening < and tag name', '',
	    "<HTML><HEAD>< TITLE>title</TITLE></HEAD>\n".
	    "<BODY>this is the body</BODY></HTML>",
	    1, 'leading-whitespace');

&ExpectFAIL('no TITLE element in HEAD', '',
	    "<HTML>\n<HEAD></HEAD>\n<BODY>this is the body</BODY>\n</HTML>",
	    2, 'empty-container',
	    2, 'require-head');

&ExpectFAIL('unclosed TITLE in HEAD', '',
	    "<HTML>\n<HEAD><TITLE></HEAD>\n".
	    "<BODY>this is the body</BODY>\n</HTML>",
	    2, 'unclosed-element');

&ExpectFAIL('bad style to use "here" as anchor text', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY><A HREF=\"foo.html\">here</A></BODY>\n</HTML>",
	    3, 'here-anchor');

&ExpectFAIL('mis-matched heading tags <H1> .. </H2>', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY><H1>title</H2></BODY>\n</HTML>",
	    3, 'mis-match', 3, 'unclosed-element');

&ExpectFAIL('obsolete element', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY><XMP>foobar()</XMP></BODY></HTML>",
	    3, 'obsolete');

&ExpectFAIL('illegal attribute in B element', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY><B FOO>foobar</B></BODY></HTML>",
	    3, 'unknown-attribute');

&ExpectFAIL('empty tag: <>', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY><>this is the body</BODY></HTML>",
	    3, 'unknown-element');

&ExpectFAIL('netscape tags *without* netscape enabled', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY BGCOLOR=\"#ffffff\">\n".
	    "<CENTER>centered text</CENTER>\n".
	    "<BLINK>blinking text</BLINK>\n".
	    "<FONT SIZE=+1>larger font size text</FONT>\n".
	    "</BODY></HTML>",
	    3, 'netscape-attribute',	# BGCOLOR attribute for BODY
	    4, 'netscape-markup',	# one for the opening tag of CENTER
	    4, 'netscape-markup',	# one for the closing tag of CENTER
	    5, 'netscape-markup',
	    5, 'netscape-markup',
	    6, 'netscape-markup',
	    6, 'netscape-markup');

&ExpectOK('netscape tags *with* netscape enabled', '-x netscape',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY BGCOLOR=\"#ffffff\">\n".
	    "<CENTER>centered text</CENTER>\n".
	    "<BLINK>blinking text</BLINK>\n".
	    "<FONT SIZE=+1>larger font size text</FONT>\n".
	    "</BODY></HTML>");

&ExpectFAIL('not allowed to nest FORM elements', '',
	    "<HTML>\n<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<FORM METHOD=post ACTION=\"http://www.khoral.com/foo\">\n".
	    "<FORM METHOD=post ACTION=\"http://www.khoral.com/foo\">\n".
	    "This is inside the nested form".
	    "</FORM>\n".
	    "</FORM></BODY></HTML>",
	    5, 'nested-element');

&ExpectFAIL('CAPTION element appearing outside of TABLE or FIG', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<TABLE><CAPTION>legal use of caption</CAPTION></TABLE>\n".
	    "<FIG><CAPTION>legal use of caption</CAPTION></FIG>\n".
	    "<CAPTION>this is an invalid use of caption</CAPTION>\n".
	    "</BODY></HTML>",
	    5, 'required-context');

&ExpectFAIL('LH element must be used in UL, OL or DL', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<UL><LH>unordered list heading</LH></UL>\n".
	    "<OL><LH>ordered list heading</LH></OL>\n".
	    "<DL><LH>definition list heading</LH></DL>\n".
	    "<LH>illegal use of list heading</LH>\n".
	    "</BODY></HTML>",
	    6, 'required-context',
	    6, 'must-follow');

&ExpectFAIL('LI element must be used in DIR, MENU, OL, OL or UL', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<DIR><LI>legal list item in DIR</DIR>\n".
	    "<MENU><LI>legal list item in MENU</MENU>\n".
	    "<OL><LI>legal list item in OL</OL>\n".
	    "<UL><LI>legal list item in UL</UL>\n".
	    "<LI>illegal list item\n".
	    "</BODY></HTML>",
	    7, 'required-context');

&ExpectFAIL('unclosed comment', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<!-- this is an unclosed comment >\n".
	    "</BODY></HTML>",
	    3, 'unclosed-comment');

&ExpectFAIL('use of physical font markup', '-e physical-font',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<B>This is bold text</B>\n".
	    "<STRONG>This is strong text</STRONG>\n".
	    "</BODY></HTML>",
	    3, 'physical-font');

&ExpectFAIL('repeated attribute', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<IMG SRC=\"foo.gif\" SRC=\"foo.gif\" ALT=\"alt text\">\n".
	    "</BODY></HTML>",
	    3, 'repeated-attribute');

&ExpectFAIL('no HTML tags around document, last thing is valid comment', '',
	    "<HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>this is the body</BODY>\n".
	    "<!-- this is a valid comment -->\n",
	    1, 'html-outer',
	    1, 'must-follow');

&ExpectFAIL('spurious text between HEAD and BODY elements', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "Should not put any text here!\n".
	    "<BODY>this is the body</BODY></HTML>\n",
	    3, 'must-follow');

&ExpectFAIL('list heading appears after first list element', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD><BODY>\n".
	    "<UL>\n".
	    "<LI>text of first list item</LI>\n".
	    "<LH>this is the list heading</LH>\n".
	    "</UL>\n".
	    "</BODY></HTML>\n",
	    4, 'must-follow');

&ExpectFAIL('empty title element', '',
	    "<HTML><HEAD><TITLE></TITLE></HEAD>\n".
	    "<BODY>this is the body</BODY></HTML>\n",
	    1, 'empty-container');

&ExpectFAIL('empty list element', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<UL>\n".
	    "<LI>this is the first element\n".
	    "<LI>\n".
	    "<LI>this is the third or second element...\n".
	    "</UL>\n".
	    "</BODY></HTML>",
	    5, 'empty-container');

&ExpectFAIL('attributes on closing tag', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY>\n".
	    "<A NAME=\"foobar\">bleh</A NAME=\"foobar\">\n".
	    "</BODY></HTML>\n",
	    3, 'closing-attribute');

&ExpectFAIL('use of netscape attributes without netscape extension', '',
	    "<HTML><HEAD><TITLE>title</TITLE></HEAD>\n".
	    "<BODY BGCOLOR=\"#fefefe\">\n".
	    "Nothing exciting in the body\n".
	    "</BODY></HTML>\n",
	    2, 'netscape-attribute');

&ExpectFAIL("use of ' as attribute value delimiter", '',
	  "<HTML><HEAD><TITLE>foo</TITLE></HEAD><BODY>\n".
	  "<IMG SRC = foo.gif ALT='alt text'>\n".
	  "</BODY></HTML>\n",
	  2, 'attribute-delimiter');

&WeblintTestEnd();


#========================================================================
# Function:	ExpectOK
# Purpose:	This is the high-level interface to the checker.  It takes
#		a file and checks for fluff.
#========================================================================
sub ExpectOK
{
   local($description, $flags, $html) = @_;
   local(@results);


   &NextTest($description);
   &CreateFile($html) || die "Failed to create working file ($filename): $!\n";
   @results = &RunWeblint($flags);
   if (@results == 0)
   {
      &TestPasses();
   }
   else
   {
      &TestFails($html, @results);
   }
}


#========================================================================
# Function:	ExpectFAIL
# Purpose:	A test which we expect weblint to complain about.
#		We pass in one or more expected errors.
#========================================================================
sub ExpectFAIL
{
   local($description, $flags, $html, @expected) = @_;
   local(@results);
   local($i, $j);


   &NextTest($description);
   &CreateFile($html) || die "Failed to create working file ($filename): $!\n";
   @results = &RunWeblint($flags);

   if (@results == 0)
   {
      &TestFails($html);
      return;
   }

   OUTER: for ($i=0; $i < $#expected; $i += 2)
   {
      INNER: for ($j = 0; $j < $#results; $j += 2)
      {
	 if ($results[$j] == $expected[$i] &&
	     $results[$j+1] eq $expected[$i+1])
	 {
	    splice(@results, $j, 2);
	    next OUTER;
	 }
      }
   }

   if (@results == 0)
   {
      &TestPasses();
   }
   else
   {
      &TestFails($html, @results);
   }
}


#========================================================================
# Function:	RunWeblint
# Purpose:	This function runs weblint and parses the output.
#		The results from weblint are passed back in an array.
#========================================================================
sub RunWeblint
{
   local($flags) = @_;
   local(*OUTPUT);
   local(@results);


   open(OUTPUT, "./weblint -t $flags $filename |") || do
   {
      die "Failed to create pipe from weblint: $!\n";
   };
   while (<OUTPUT>)
   {
      next if /^$/;
      chop;
      ($repfile, $line, $wid) = split(/:/);
      push(@results, $line, $wid);
   }
   close OUTPUT;
   $status = ($? >> 8);

   return @results;
}


#========================================================================
# Function:	CreateFile
# Purpose:	Create sample html file from text string.
#========================================================================
sub CreateFile
{
   local($html) = @_;
   local(*FILE);


   open(FILE, "> $filename") || return undef;
   print FILE $html."\n";
   close FILE;

   1;
}


#========================================================================
# Function:	WeblintTestInitialize()
# Purpose:	Initialize global variables and open log file.
#========================================================================
sub WeblintTestInitialize
{
   $TMPDIR   = $ENV{'TMPDIR'} || '/usr/tmp';
   $WORKDIR  = "$TMPDIR/weblint-test.$$";
   mkdir($WORKDIR, 0755) || do
   {
      die "Failed to create working directory $WORKDIR: $!\n";
   };

   $filename = $WORKDIR.'/'.$FILENAME;
   $testID   = 0;
   $failCount = 0;
   $passCount = 0;
   chop($WEBLINTVERSION = `./weblint -v 2>&1`);

   open(LOGFILE, "> $LOGFILE") || die "Can't write logfile $LOGFILE: $!\n";

   print LOGFILE "Weblint Testsuite:\n";
   print LOGFILE "    Weblint Version:   $WEBLINTVERSION\n";
   print LOGFILE "    Testsuite Version: $VERSION\n";
   print LOGFILE '=' x 76, "\n";

   print STDERR "Running weblint testsuite:\n";
   print STDERR "    Weblint Version:   $WEBLINTVERSION\n";
   print STDERR "    Testsuite Version: $VERSION\n";
}

#========================================================================
# Function:	WeblintTestEnd()
# Purpose:	Generate summary in logfile, close logfile, then
#		clean up working files and directory.
#========================================================================
sub WeblintTestEnd
{
   print LOGFILE '=' x 76, "\n";
   print LOGFILE "Number of Passes:   $passCount\n";
   print LOGFILE "Number of Failures: $failCount\n";
   close LOGFILE;

   print STDERR '-' x 76, "\n";
   print STDERR "Number of Passes:   $passCount\n";
   print STDERR "Number of Failures: $failCount\n";

   unlink $filename;
   rmdir $WORKDIR;
}

#========================================================================
# Function:	NextTest()
# Purpose:	Introduce a new test -- increment test id, write
#		separator and test information to log file.
#========================================================================
sub NextTest
{
   local($description) = @_;


   ++$testID;
   print LOGFILE '-' x 76, "\n";
   $testDescription = $description;
}

#========================================================================
# Function:	TestPasses()
# Purpose:	The current test passed.  Write result to logfile, and
#		increment the count of successful tests.
#========================================================================
sub TestPasses
{
   printf LOGFILE ("%3d %s%s%s", $testID, $testDescription,
		   ' ' x (68 - length($testDescription)), "PASS\n");
   printf STDERR "%3d: pass (%s)\n", $testID, $testDescription;
   ++$passCount;
}

#========================================================================
# Function:	TestFails()
# Purpose:	The current test failed.  Write result to logfile,
#		including the html which failed, and the output from weblint.
#========================================================================
sub TestFails
{
   local($html, @results) = @_;


   printf STDERR "%3d: FAIL (%s)\n", $testID, $testDescription;
   printf LOGFILE ("%3d %s%s%s", $testID, $testDescription,
		   ' ' x (68 - length($testDescription)), "FAIL\n");

   $html =~ s/\n/\n    /g;
   print LOGFILE "\n  HTML:\n    $html\n\n";
   print LOGFILE "  WEBLINT OUTPUT:\n";
   while (@results > 1)
   {
      ($line, $wid) = splice(@results, 0, 2);
      print LOGFILE "    line $line: $wid\n";
   }
   print LOGFILE "\n";
   ++$failCount;
}
